
package FieldBlockBuilderImporter::Plugin;
use strict;

sub list_actions_export {
    my ( $app ) = @_;
    my $plugin = MT->component('fieldblockbuilderimporter');
    my $user = $app->user;

    $app->validate_magic or return $app->error('Invalid request.');

    unless ($user->can_do('export_fieldblockbuilder_block_def')) {
        return $app->return_to_dashboard( permission => 1 );
    }

    $app->setup_filtered_ids if $app->param('all_selected');

    my @ids = $app->param('id');

    my @column_names = qw( 
        basename label field_defs_json
        use_custom_edit_template use_custom_view_template
        edit_template edit_template_css edit_template_js view_template
        is_only_used_in_sub_blocks maximum_usage
    );

    eval {
        my @tl = MT::Util::offset_time_list( time );
        my $filename = sprintf('fieldblockbuilder_block_def_%04d%02d%02d%02d%02d%02d.csv', $tl[5]+1900, $tl[4]+1, @tl[3,2,1,0]);

        my $iter = $app->model('fieldblockbuilder_block_def')->load_iter({ id => \@ids }, { sort => 'id' });

        my $csv = Text::CSV->new({ binary => 1 });

        $app->{'no_print_body'} = 1;
        $app->set_header('Content-Disposition' => "attachment; filename=" . MT::Util::encode_url($filename));
        $app->set_header('Pragma' => '') if $app->is_secure;
        $app->send_http_header('text/csv');

        $app->print("\xEF\xBB\xBF");

        if ($csv->combine( @column_names )) {
            $app->print($csv->string . "\n");
        }

        while (my $block_def = $iter->()) {
            my @cols = ();

            foreach my $col_name (@column_names) {
                push(@cols, $block_def->$col_name);
            }

            if ($csv->combine(@cols)) {
                $app->print($csv->string . "\n");
            }
        }
    };

    if ( $@ ) {
        my $message = $@;
        die $message;
    }
}

sub mode_start_fieldblockbuilder_block_def_import {
    my $app = shift;
    my $plugin = MT->component( 'fieldblockbuilderimporter' );
    my $user = $app->user;

    unless ($user->can_do('import_fieldblockbuilder_block_def')) {
        return $app->return_to_dashboard( permission => 1 );
    }

    my $params;
    $params->{'screen_group'} = 'fieldblockbuilder_block_def';

    return $plugin->load_tmpl('fieldblockbuilder_block_def_import_start.tmpl', $params);
}

sub mode_fieldblockbuilder_block_def_import {
    my $app = shift;
    my $plugin = MT->component( 'fieldblockbuilderimporter' );
    my $user = $app->user;
    my $blog = $app->blog;

    $app->validate_magic or return;

    unless ($user->can_do('import_fieldblockbuilder_block_def')) {
        return $app->return_to_dashboard( permission => 1 );
    }

    my $fh = $app->param->upload('file');
    unless ($fh) {
        return $plugin->load_tmpl('fieldblockbuilder_block_def_import_start.tmpl', { 
            'error_file_required' => 1,
            'screen_group' => 'fieldblockbuilder_block_def',
        });
    }

    my $class = MT->model('fieldblockbuilder_block_def');

    my $csv = Text::CSV->new({ binary => 1 });
    my @results;

    my $headers = $csv->getline($fh);
    $headers->[0] =~ s/^\x{FEFF}//;

    my %index_by_colname;
    my $is_valid = 1;
    for (my $i = 0; $i < @$headers; $i++) {
        my $col_name = $headers->[$i];
        $is_valid = 0 unless $class->has_column($col_name);
        $index_by_colname{$col_name} = $i;
    }

    if (@$headers == 0 || !$is_valid) {
        return $plugin->load_tmpl('fieldblockbuilder_block_def_import_start.tmpl', { 
            'error_invalid_file' => 1,
            'screen_group' => 'fieldblockbuilder_block_def',
        });
    }

    my $col_basename = $index_by_colname{'basename'};
    my $col_label = $index_by_colname{'label'};

    while (my $cols = $csv->getline($fh)) {
        my $basename = $cols->[$col_basename];

        my %result;
        $result{'basename'} = $basename;
        $result{'label'} = $cols->[$col_label];

        if ($basename) {
            my $block_def = $class->get_by_key({ 'blog_id' => $blog->id, 'basename' => $basename });

            $result{'is_update'} = ($block_def->id ne '');

            for (my $i = 0; $i < @$headers; $i++) {
                my $col_name = $headers->[$i];
                next if $col_name eq 'id';
                my $value = $cols->[$i];
                $block_def->column($col_name, $value);
            }

            my @errors = $class->validate_field_defs_json($block_def->field_defs_json);

            if (@errors == 0) {
                $block_def->save() or $result{'error'} = $plugin->translate('Failed to save block definition.') .
                                                         "\n" . $block_def->errstr;
            } else {
                $result{'error'} = $plugin->translate('field_defs_json is invalid.') .
                                    "\n" . MT::Util::to_json(\@errors);
            }

            $result{'id'} = $block_def->id;
        } else {
            $result{'error'} = $plugin->translate('Basename required.');
        }

        push(@results, \%result);
    }

    $app->log({
        'message' => $plugin->translate('Block definition has been imported.'),
        'metadata' => MT::Util::to_json(\@results),
    });

    my $params;
    $params->{'has_error'} = (grep { $_->{'error'} ne ''} @results) > 0;
    $params->{'results'} = \@results;
    $params->{'screen_group'} = 'fieldblockbuilder_block_def';

    return $plugin->load_tmpl('fieldblockbuilder_block_def_import_result.tmpl', $params);
}

1;
